==> sish-2.11.0.tar.gz: sish-2.11.0/httpmuxer/https.go <==
package httpmuxer

import (
	"encoding/base64"
	"fmt"
	"log"
	"net"

	"github.com/antoniomika/sish/utils"
	"github.com/spf13/viper"
	"github.com/vulcand/oxy/roundrobin"
)

type proxyListener struct {
	Listener net.Listener
	Holder   *utils.TCPHolder
	State    *utils.State
}

func (pL *proxyListener) Accept() (net.Conn, error) {
	for {
		cl, err := pL.Listener.Accept()
		if err != nil {
			return nil, err
		}

		clientRemote, _, err := net.SplitHostPort(cl.RemoteAddr().String())

		if err != nil || pL.State.IPFilter.Blocked(clientRemote) {
			cl.Close()
			continue
		}

		tlsHello, buf, teeConn, peekErr := utils.PeekTLSHello(cl)
		if peekErr != nil && tlsHello == nil {
			return teeConn, nil
		}

		balancerName := tlsHello.ServerName
		if balancerName == "" {
			return teeConn, nil
		}

		balancer, ok := pL.Holder.Balancers.Load(balancerName)
		if !ok {
			pL.Holder.Balancers.Range(func(n string, b *roundrobin.RoundRobin) bool {
				if utils.MatchesWildcardHost(balancerName, n) {
					balancer = b
					return false
				}
				return true
			})

			if balancer == nil {
				return teeConn, nil
			}
		}

		connectionLocation, err := balancer.NextServer()
		if err != nil {
			log.Println("Unable to load connection location:", err)
			cl.Close()
			continue
		}

		host, err := base64.StdEncoding.DecodeString(connectionLocation.Host)
		if err != nil {
			log.Println("Unable to decode connection location:", err)
			cl.Close()
			continue
		}

		hostAddr := string(host)

		logLine := fmt.Sprintf("Accepted connection from %s -> %s", cl.RemoteAddr().String(), cl.LocalAddr().String())
		log.Println(logLine)

		if viper.GetBool("log-to-client") {
			pL.Holder.SSHConnections.Range(func(key string, sshConn *utils.SSHConnection) bool {
				sshConn.Listeners.Range(func(listenerAddr string, val net.Listener) bool {
					if listenerAddr == hostAddr {
						sshConn.SendMessage(logLine, true)

						return false
					}

					return true
				})

				return true
			})
		}

		conn, err := net.Dial("unix", hostAddr)
		if err != nil {
			log.Println("Error connecting to tcp balancer:", err)
			cl.Close()
			continue
		}

		_, err = conn.Write(buf.Bytes())
		if err != nil {
			log.Println("Unable to write to conn:", err)
			cl.Close()
			continue
		}

		go utils.CopyBoth(conn, cl)
	}
}

func (pL *proxyListener) Close() error {
	return pL.Listener.Close()
}

// Addr returns the listener's network address.
func (pL *proxyListener) Addr() net.Addr {
	return pL.Listener.Addr()
}

==> sish-2.16.1.tar.gz: sish-2.16.1/httpmuxer/https.go <==
package httpmuxer

import (
	"encoding/base64"
	"fmt"
	"log"
	"net"

	"github.com/antoniomika/sish/utils"
	"github.com/spf13/viper"
	"github.com/vulcand/oxy/roundrobin"
)

type proxyListener struct {
	Listener net.Listener
	Holder   *utils.TCPHolder
	State    *utils.State
}

func (pL *proxyListener) Accept() (net.Conn, error) {
	for {
		cl, err := pL.Listener.Accept()
		if err != nil {
			return nil, err
		}

		clientRemote, _, err := net.SplitHostPort(cl.RemoteAddr().String())

		if err != nil || pL.State.IPFilter.Blocked(clientRemote) {
			cl.Close()
			continue
		}

		tlsHello, teeConn, _ := utils.PeekTLSHello(cl)
		if tlsHello == nil {
			return teeConn, nil
		}

		balancerName := tlsHello.ServerName
		if balancerName == "" {
			return teeConn, nil
		}

		balancer, ok := pL.Holder.Balancers.Load(balancerName)
		if !ok {
			pL.Holder.Balancers.Range(func(n string, b *roundrobin.RoundRobin) bool {
				if utils.MatchesWildcardHost(balancerName, n) {
					balancer = b
					return false
				}
				return true
			})

			if balancer == nil {
				return teeConn, nil
			}
		}

		connectionLocation, err := balancer.NextServer()
		if err != nil {
			log.Println("Unable to load connection location:", err)
			teeConn.Close()
			continue
		}

		host, err := base64.StdEncoding.DecodeString(connectionLocation.Host)
		if err != nil {
			log.Println("Unable to decode connection location:", err)
			teeConn.Close()
			continue
		}

		hostAddr := string(host)

		logLine := fmt.Sprintf("Accepted connection from %s -> %s", teeConn.RemoteAddr().String(), teeConn.LocalAddr().String())
		log.Println(logLine)

		if viper.GetBool("log-to-client") {
			pL.Holder.SSHConnections.Range(func(key string, sshConn *utils.SSHConnection) bool {
				sshConn.Listeners.Range(func(listenerAddr string, val net.Listener) bool {
					if listenerAddr == hostAddr {
						sshConn.SendMessage(logLine, true)

						return false
					}

					return true
				})

				return true
			})
		}

		conn, err := net.Dial("unix", hostAddr)
		if err != nil {
			log.Println("Error connecting to tcp balancer:", err)
			teeConn.Close()
			continue
		}

		go utils.CopyBoth(conn, teeConn)
	}
}

func (pL *proxyListener) Close() error {
	return pL.Listener.Close()
}

// Addr returns the listener's network address.
func (pL *proxyListener) Addr() net.Addr {
	return pL.Listener.Addr()
}
